/*
 * $Id: gamemanager.c,v 1.2 2002/03/13 16:09:58 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Game lifecycle manager.
 *
 * @version $Revision: 1.2 $
 */
#include <stdlib.h>
#include <piece.h>

#include "sbrg.h"
#include "gamemanager.h"
#include "render.h"
#include "sincos.h"
#include "usb.h"

#define MOTOR_MAX 4
static Motor motor[MOTOR_MAX];

#define FRAG_MAX 20
static Frag frag[FRAG_MAX];
static int fragIdx = FRAG_MAX;

#define DEFAULT_EYE_DIST 112
static Vector eyePos;
static int eyeDist, eyeDeg;

#define WALL_HEIGHT 384
#define FIELD_WIDTH 49152
#define DOT_INTERVAL 6144

#define DOT_COLOR 2
#define FIELD_WALL_COLOR 2

static int fwIdx[4][4] = {{0,1,2,3}, {2,3,6,7}, {6,7,4,5}, {4,5,0,1}};

// Make game field.
static void initField() {
  int x, y, i;
  Point *p;
  Point *lp[8];
  Line *l;
  Dot *d;
  for ( x = -FIELD_WIDTH/2+DOT_INTERVAL ; x < FIELD_WIDTH/2 ; x += DOT_INTERVAL ) {
    for ( y = -FIELD_WIDTH/2+DOT_INTERVAL ; y < FIELD_WIDTH/2 ; y += DOT_INTERVAL ) {
      p = getPointInstance();
      p->a.x = x; p->a.y = 0; p->a.z = y;
      d = getDotInstance();
      d->p = p;
      d->color = DOT_COLOR;
      d->size = 3;
    }
  }
  i = 0;
  for ( x = -FIELD_WIDTH/2 ; x <= FIELD_WIDTH/2 ; x += FIELD_WIDTH ) {
    for ( y = -FIELD_WIDTH/2 ; y <= FIELD_WIDTH/2 ; y += FIELD_WIDTH ) {
      lp[i] = getPointInstance();
      lp[i+1] = getPointInstance();
      lp[i]->a.x = x; lp[i]->a.y = 0; lp[i]->a.z = y;
      lp[i+1]->a.x = x; lp[i+1]->a.y = WALL_HEIGHT*2; lp[i+1]->a.z = y;
      i += 2;
    }
  }
  for ( i=0 ; i<4 ; i++ ) {
    l = getLineInstance();
    l->sp = lp[fwIdx[i][0]]; l->ep = lp[fwIdx[i][1]]; l->color = FIELD_WALL_COLOR;
    l = getLineInstance();
    l->sp = lp[fwIdx[i][0]]; l->ep = lp[fwIdx[i][2]]; l->color = FIELD_WALL_COLOR;
    l = getLineInstance();
    l->sp = lp[fwIdx[i][1]]; l->ep = lp[fwIdx[i][3]]; l->color = FIELD_WALL_COLOR;
  }
}

void initGamemanager() {
  int i, j;
  Motor *mt;
  Wall *wl;
  for ( i=0 ; i<MOTOR_MAX ; i++ ) {
    mt = &(motor[i]);
    for ( j=0 ; j<WALL_MAX ; j++ ) {
      wl = &(mt->wall[j]);
      wl->pt[0] = getPointInstance();
      wl->pt[1] = getPointInstance();
      if ( j > 0 ) {
	wl = &(mt->wall[j-1]);
	wl->ln[0] = getLineInstance();
	wl->ln[1] = getLineInstance();
	wl->ln[0]->sp = wl->pt[0];
	wl->ln[0]->ep = mt->wall[j].pt[0];
	wl->ln[1]->sp = wl->pt[1];
	wl->ln[1]->ep = mt->wall[j].pt[1];
      }
    }
  }
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    frag[i].dot = getDotInstance();
    frag[i].dot->p = getPointInstance();
  }
  initField();
}

#define MOTOR_SPEED 384

// Initialize motor's position.
static void initMotor(Motor *mt, int x, int y, int d, int idx) {
  mt->x = x; mt->y = y;
  mt->d = d;
  mt->speed = MOTOR_SPEED;
  mt->com = 0;
  mt->wall[0].pt[0]->a.x = mt->wall[0].pt[1]->a.x = mt->x;
  mt->wall[0].pt[0]->a.z = mt->wall[0].pt[1]->a.z = mt->y;
  mt->wall[0].pt[0]->a.y = 0;
  mt->wall[0].pt[1]->a.y = WALL_HEIGHT;
  mt->wall[1].pt[0]->a.x = mt->wall[1].pt[1]->a.x = mt->x;
  mt->wall[1].pt[0]->a.z = mt->wall[1].pt[1]->a.z = mt->y;
  mt->wall[1].pt[0]->a.y = 0;
  mt->wall[1].pt[1]->a.y = WALL_HEIGHT;
  mt->wall[0].ln[0]->sp = mt->wall[0].pt[0];
  mt->wall[0].ln[0]->ep = mt->wall[1].pt[0];
  mt->wall[0].ln[0]->color = 3;
  mt->wall[0].ln[1]->sp = mt->wall[0].pt[1];
  mt->wall[0].ln[1]->ep = mt->wall[1].pt[1];
  mt->wall[0].ln[1]->color = 3;
  mt->wall[0].d = d;
  mt->wallIdx = 1; mt->wallLoopFlag = 0;
  mt->deadCnt = 0;

  setWallUSB(idx, 0, x, y, d);
}

// Calc a distance to the wall.
static int calcWallDist(int x, int y, int d, int mtIdx) {
  int i, j;
  Motor *mt;
  Wall *wl;
  Point *p1, *p2;
  int dst, td;
  int wc;
  
  // Distance from a field wall.
  switch ( d ) {
  case 0:
    dst = (FIELD_WIDTH>>1)-y;
    break;
  case 1:
    dst = (FIELD_WIDTH>>1)-x;
    break;
  case 2:
    dst = y+(FIELD_WIDTH>>1);
    break;
  case 3:
    dst = x+(FIELD_WIDTH>>1);
    break;
  default:
    dst = 999999999;
    break;
  }
  for ( i=0 ; i<MOTOR_MAX ; i++ ) {
    if ( motor[i].speed == NOT_EXIST || motor[i].deadCnt > 0 ) continue;
    mt = &(motor[i]);
    if ( mt->wallLoopFlag ) wc = WALL_MAX-1;
    else wc = mt->wallIdx;
    for ( j=0 ; j<wc ; j++ ) {
      if ( i == mtIdx && j == mt->wallIdx-1 ) continue; // Don't check the wake wall.
      wl = &(mt->wall[j]);
      if ( wl->d == -1 ) continue;
      p1 = wl->pt[0]; p2 = mt->wall[j+1].pt[0];
      if ( (d&1) == (wl->d&1) ) {
	switch ( wl->d ) {
	case 0:
	  if ( p1->a.x == x ) {
	    if ( d == 0 ) {
	      if ( y > p2->a.z ) continue;
	      td = p1->a.z - y;
	    } else {
	      if ( y < p1->a.z ) continue;
	      td = y- p2->a.z - y;
	    }
	  } else {
	    continue;
	  }
	  break;
	case 1:
	  if ( p1->a.z == y ) {
	    if ( d == 1 ) {
	      if ( x > p2->a.x ) continue;
	      td = p1->a.x - x;
	    } else {
	      if ( x < p1->a.x ) continue;
	      td = x - p2->a.x;
	    }
	  } else {
	    continue;
	  }
	  break;
	case 2:
	  if ( p1->a.x == x ) {
	    if ( d == 0 ) {
	      if ( y > p1->a.z ) continue;
	      td = p2->a.z - y;
	    } else {
	      if ( y < p2->a.z ) continue;
	      td = y - p1->a.z;
	    }
	  } else {
	    continue;
	  }
	  break;
	case 3:
	  if ( p1->a.z == y ) {
	    if ( d == 1 ) {
	      if ( x > p1->a.x ) continue;
	      td = p2->a.x - x;
	    } else {
	      if ( x < p2->a.x ) continue;
	      td = x - p1->a.x;
	    }
	  } else {
	    continue;
	  }
	  break;
	default:
	  td = 999999999;
	  break;
	}
	if ( td < 0 ) td = 0;
      } else {
	switch ( wl->d ) {
	case 0:
	  if ( d == 1 ) {
	    if ( x > p1->a.x ) continue;
	  } else {
	    if ( x < p1->a.x ) continue;
	  }
	  if ( y >= p1->a.z && y <= p2->a.z ) {
	    td = p1->a.x - x;
	  } else {
	    continue;
	  }
	  break;
	case 1:
	  if ( d == 0 ) {
	    if ( y > p1->a.z ) continue;
	  } else {
	    if ( y < p1->a.z ) continue;
	  }
	  if ( x >= p1->a.x && x <= p2->a.x ) {
	    td = p1->a.z - y;
	  } else {
	    continue;
	  }
	  break;
	case 2:
	  if ( d == 1 ) {
	    if ( x > p1->a.x ) continue;
	  } else {
	    if ( x < p1->a.x ) continue;
	  }
	  if ( y >= p2->a.z && y <= p1->a.z ) {
	    td = p1->a.x - x;
	  } else {
	    continue;
	  }
	  break;
	case 3:
	  if ( d == 0 ) {
	    if ( y > p1->a.z ) continue;
	  } else {
	    if ( y < p1->a.z ) continue;
	  }
	  if ( x >= p2->a.x && x <= p1->a.x ) {
	    td = p1->a.z - y;
	  } else {
	    continue;
	  }
	  break;
	default:
	  td = 999999999;
	  break;
	}
	if ( td < 0 ) td = -td;
      }
      if ( td < dst ) dst = td;
    }
  }
  return dst;
}

static unsigned char *mtrBgm[] = {
  SE_BGM1, SE_BGM2, SE_BGM3, SE_BGM4, SE_BGM5, SE_BGM6, SE_BGM7, SE_BGM8, 
};
static int bgmNum = 0;

// Initialize computer motor.
static void setComMotor(Motor *mt, int rank, int idx) {
  int x, y, dst, d = 0, maxDst;
  int i, j;
  for ( i=0 ; i<16 ; i++ ) {
    x = randNS(FIELD_WIDTH/MOTOR_SPEED/3)*MOTOR_SPEED;
    y = randNS(FIELD_WIDTH/MOTOR_SPEED/3)*MOTOR_SPEED;
    if ( absN(x-motor[0].x) + absN(y-motor[0].y) < FIELD_WIDTH/4 ) continue;
    maxDst = 0;
    for ( j=0 ; j<4 ; j++ ) {
      dst = calcWallDist(x, y, j, -1);
      if ( dst > maxDst ) {
	maxDst = dst; d = j;
      }
    }
    if ( maxDst > FIELD_WIDTH/10 ) break;
  }
  initMotor(mt, x, y, d, idx);
  mt->senseInterval = rank/24 - randN(3);
  if ( mt->senseInterval < 2 ) mt->senseInterval = 2;
  mt->senseRange = (256-rank)*48 - randN(2048);
  if ( mt->senseRange < 1 ) mt->senseRange = 1;
  mt->margin = MOTOR_SPEED*2 + randN(4092);
  mt->stability = (16 + randN(64)) / mt->senseInterval;
  mt->com = 1;
  mt->bgm = mtrBgm[bgmNum];
  bgmNum++; bgmNum &= 7;
}

#define SHIELD_MAX 100<<8
#define REPAIR_DEFAULT 600

static int gameCnt;
// Flags that are set when the motor crashed.
static int dstWlSet, dstWlFlag;
// Player invicible counter, shield, an amount of shield repair.
static int ivcCnt, shield, repair;
static int demoFlag;
static int playerDeadCnt;
static int score;

// Clear all motors and fragments.
static void initMotorFrag() {
  int i, j;
  for ( i=0 ; i<MOTOR_MAX ; i++ ) {
    motor[i].speed = NOT_EXIST;
    motor[i].wallIdx = 0;
    motor[i].wallLoopFlag = 0;
    for ( j=0 ; j<WALL_MAX ; j++ ) {
      motor[i].wall[j].d = -1;
      motor[i].wall[j].pt[0]->a.x = NOT_EXIST;
      motor[i].wall[j].pt[1]->a.x = NOT_EXIST;
      motor[i].wall[j].ln[0]->color = NOT_EXIST;
      motor[i].wall[j].ln[1]->color = NOT_EXIST;
    }
  }
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    frag[i].cnt = 0;
  }
  resetGameUSB();
}

// Start game.
void initGame() {
  int i;
  initMotorFrag();
  initMotor(&(motor[0]), FIELD_WIDTH>>2, FIELD_WIDTH>>2, 2, 0);
  for ( i=1 ; i<MOTOR_MAX ; i++ ) {
    setComMotor(&(motor[i]), 256, i);
  }
  gameCnt = 0;
  score = 0;
  playerDeadCnt = -1;
  dstWlSet = 0; dstWlFlag = 0;
  ivcCnt = 32; shield = SHIELD_MAX; repair = REPAIR_DEFAULT;
  pceWaveSetChAtt(0, 8);
  demoFlag = 0;

  eyeDist = DEFAULT_EYE_DIST + 256;
  eyeDeg = 12;
}

// Start demo.
void initGameDemo() {
  int i;
  initMotorFrag();
  //initMotor(&(motor[0]), 0, 0, randN(32), 0);
  motor[0].x = motor[0].y = 0;
  setComMotor(&(motor[0]), randN(32), 0);
  for ( i=1 ; i<MOTOR_MAX ; i++ ) {
    setComMotor(&(motor[i]), 96+randN(128), i);
  }
  gameCnt = 0;
  playerDeadCnt = -1;
  dstWlSet = 0; dstWlFlag = 0;
  pceWaveSetChAtt(0, 20);
  demoFlag = 1;

  eyeDist = DEFAULT_EYE_DIST;
  eyeDeg = 0;
}

static void addFrag(int x, int y, int z, 
		    int mx, int my, int mz, 
		    int cnt, int size) {
  Frag *fr;
  Point *p;
  fragIdx--; if ( fragIdx < 0 ) fragIdx = FRAG_MAX-1;
  fr = &(frag[fragIdx]);
  p = fr->dot->p;
  p->a.x = x; p->a.y = y; p->a.z = z;
  fr->dot->color = 3;
  fr->mx = mx; fr->my = my; fr->mz = mz; 
  fr->cnt = cnt;
  fr->size = size;
}

#define CRASH_RANGE 10240

// Destroy walls when the motor crashed into a wall.
static void destroyWalls(int x, int y) {
  int i, j;
  Motor *mt;
  Point *p1, *p2;
  Wall *wl;
  int wc;
  int dst = 999999999;
  dstWlSet = 1;
  for ( i=0 ; i<MOTOR_MAX ; i++ ) {
    if ( motor[i].speed == NOT_EXIST || motor[i].deadCnt > 0 ) continue;
    mt = &(motor[i]);
    if ( mt->wallLoopFlag ) wc = WALL_MAX-1;
    else wc = mt->wallIdx;
    for ( j=0 ; j<wc ; j++ ) {
      wl = &(mt->wall[j]);
      if ( wl->d == -1 ) continue;
      p1 = wl->pt[0]; p2 = mt->wall[j+1].pt[0];
      switch ( wl->d ) {
      case 0:
	if ( y > p1->a.z ) {
	  if ( y > p2->a.z ) {
	    dst = y - p2->a.z;
	  } else {
	    dst = 0;
	  }
	} else {
	  dst = p1->a.z - y;
	}
	dst += absN(x - p1->a.x);
	break;
      case 1:
	if ( x > p1->a.x ) {
	  if ( x > p2->a.x ) {
	    dst = x - p2->a.x;
	  } else {
	    dst = 0;
	  }
	} else {
	  dst = p1->a.x - x;
	}
	dst += absN(y - p1->a.z);
	break;
      case 2:
	if ( y > p2->a.z ) {
	  if ( y > p1->a.z ) {
	    dst = y - p1->a.z;
	  } else {
	    dst = 0;
	  }
	} else {
	  dst = p2->a.z - y;
	}
	dst += absN(x - p1->a.x);
	break;
      case 3:
	if ( x > p2->a.x ) {
	  if ( x > p1->a.x ) {
	    dst = x - p1->a.x;
	  } else {
	    dst = 0;
	  }
	} else {
	  dst = p2->a.x - x;
	}
	dst += absN(y - p1->a.z);
	break;
      }
      if ( dst < CRASH_RANGE ) {
	wl->d = -1;
	wl->ln[0]->color = NOT_EXIST;
	wl->ln[1]->color = NOT_EXIST;
	destroyWallUSB(i, j);
      }
    }
  }
  for ( i=0 ; i<8+randN(8) ; i++ ) {
    addFrag(x, 0, y, randNS(1024), randN(512), randNS(1024), 8+randN(24), 5);
  }
}

// Remove walls when the motor is destroied.

#define DEAD_CNT 32
#define WALL_DOWN (WALL_HEIGHT/DEAD_CNT)

static void downWalls(Motor *mt) {
  int i, wc;
  if ( mt->wallLoopFlag ) wc = WALL_MAX-1;
  else wc = mt->wallIdx;
  for ( i=0 ; i<=wc ; i++ ) {
    mt->wall[i].pt[1]->a.y -= WALL_DOWN;
  }
}

static void clearWalls(Motor *mt) {
  int i, wc;
  Wall *wl;
  if ( mt->wallLoopFlag ) wc = WALL_MAX-1;
  else wc = mt->wallIdx;
  for ( i=0 ; i<wc ; i++ ) {
    wl = &(mt->wall[i]);
    wl->pt[0]->a.x = NOT_EXIST;
    wl->pt[1]->a.x = NOT_EXIST;
    wl->ln[0]->color = NOT_EXIST;
    wl->ln[1]->color = NOT_EXIST;
    wl->d = -1;
  }
  wl = &(mt->wall[wc]);
  wl->pt[0]->a.x = NOT_EXIST;
  wl->pt[1]->a.x = NOT_EXIST;
}


#define INVINCIBLE_TIME 12

void moveMotors(int pad) {
  int i;
  int chgDeg;
  Motor *mt;
  Wall *wl;
  int ipt;
  int dst;
  int mx = 0, my = 0;
  for ( i=0 ; i<MOTOR_MAX ; i++ ) {
    if ( motor[i].speed == NOT_EXIST ) continue;
    mt = &(motor[i]);

    // Motor is destoried.
    if ( mt->deadCnt > 0 ) {
      downWalls(mt);
      mt->deadCnt--;
      if ( mt->deadCnt <= 0 ) {
	clearWalls(mt);
	if ( mt->com ) {
	  // Set a next computer motor.
	  if ( !demoFlag ) {
	    int rank;
	    rank = gameCnt/15; if ( rank > 256 ) rank = 256;
	    setComMotor(mt, 256-rank + randN(rank/2), i);
	  } else {
	    if ( i == 0 ) {
	      setComMotor(mt, randN(32), i);
	    } else {
	      setComMotor(mt, 96+randN(128), i);
	    }
	  }
	} else {
	  mt->speed = NOT_EXIST;
	}
      }
      continue;
    }

    // Check if the motor crashed into a wall.
    dst = calcWallDist(mt->x, mt->y, mt->d, i);
    if ( dst <= 0 ) {
      if ( mt->com || ivcCnt <= 0 ) {
	// Crashed.
	if ( mt->com ) {
	  deadMotorUSB(i, mt->x, mt->y);
	  destroyWalls(mt->x, mt->y);
	  playSe(SE_CRASH, i);
	  if ( playerDeadCnt < 0 ) score++;
	  mt->deadCnt = DEAD_CNT;
	  continue;
	} else {
	  playSe(SE_DAMAGE, i);
	  if ( shield >= SHIELD_MAX && 
	       mt->x < (FIELD_WIDTH>>1) &&  mt->x > -(FIELD_WIDTH>>1) &&  
	       mt->y < (FIELD_WIDTH>>1) &&  mt->y > -(FIELD_WIDTH>>1) ) {
	    crashMotorUSB(i, mt->x, mt->y);
	    destroyWalls(mt->x, mt->y);
	    ivcCnt = INVINCIBLE_TIME;
	    shield = 0; repair *= 0.94;
	  } else {
	    deadMotorUSB(i, mt->x, mt->y);
	    destroyWalls(mt->x, mt->y);
	    mt->deadCnt = DEAD_CNT;
	    playerDeadCnt = 0;
	    continue;
	  }
	}
      } else {
	// Player is invincible.
	if ( mt->x >= (FIELD_WIDTH>>1) ||  mt->x <= -(FIELD_WIDTH>>1) ||
	     mt->y >= (FIELD_WIDTH>>1) ||  mt->y <= -(FIELD_WIDTH>>1) ) {
	  deadMotorUSB(i, mt->x, mt->y);
	  destroyWalls(mt->x, mt->y);
	  mt->deadCnt = DEAD_CNT;
	  playerDeadCnt = 0;
	  continue;
	}
      }
    } else {
      if ( !mt->com && ivcCnt > 0 ) ivcCnt--;
    }

    // Repair shield.
    if ( !mt->com && shield < SHIELD_MAX ) {
      shield += repair;
      if ( shield > SHIELD_MAX ) shield = SHIELD_MAX;
    }

    ipt = 0;
    if ( !mt->com ) {
      // Control the motor with a pad.
      if ( pad & (TRG_B | TRG_LF) ) {
	ipt |= 1;
	if ( ivcCnt <= 0 ) playSe(SE_RIGHT, 0);
      }
      if ( pad & (TRG_A | TRG_RI) ) {
	ipt |= 2;
	if ( ivcCnt <= 0 ) playSe(SE_LEFT, 0);
      }
    } else {
      // Computer control the motor.
      if ( (cnt%mt->senseInterval) == 0 ) {
	if ( dst < mt->margin || randN(mt->stability) == 0 ) {
	  int d1, d2;
	  d1 = calcWallDist(mt->x, mt->y, (mt->d+1)&3, i);
	  if ( d1 > mt->senseRange ) d1 = mt->senseRange;
	  d2 = calcWallDist(mt->x, mt->y, (mt->d-1)&3, i);
	  if ( d2 > mt->senseRange ) d2 = mt->senseRange;
	  if ( dst > mt->senseRange ) dst = mt->senseRange;
	  if ( d2 > d1 ) {
	    if ( d2 >= dst ) ipt |= 1;
	  } else if ( d1 > d2 ) {
	    if ( d1 >= dst ) ipt |= 2;
	  } else {
	    if ( d1 >= dst ) {
	      if ( i == 0 ) {
		if ( randN(2) == 0 ) ipt |= 1;
		else ipt |= 2;
	      } else { 
		switch ( mt->d ) {
		case 0:
		  if ( motor[0].x < mt->x ) ipt |= 1;
		  else ipt |= 2;
		  break;
		case 1:
		  if ( motor[0].y < mt->x ) ipt |= 2;
		  else ipt |= 1;
		  break;
		case 2:
		  if ( motor[0].x < mt->x ) ipt |= 2;
		  else ipt |= 1;
		  break;
		case 3:
		  if ( motor[0].y < mt->x ) ipt |= 1;
		  else ipt |= 2;
		  break;
		}
	      }
	    }
	  }
	}
      }
    }
    chgDeg = 0;
    if ( ipt & 1 ) {
      mt->d--; if ( mt->d < 0 ) mt->d += 4;
      chgDeg = 1;
    } else if ( ipt & 2 ) {
      mt->d++; if ( mt->d > 3 ) mt->d -= 4;
      chgDeg = 1;
    }

    // Update walls when the motor turns or some motor crashed into a wall.
    if ( chgDeg || dstWlFlag ) {
      if ( mt->wallIdx >= WALL_MAX-1 ) {
	wl = &(mt->wall[0]);
	wl->pt[0]->a.x = wl->pt[1]->a.x = mt->x;
	wl->pt[0]->a.z = wl->pt[1]->a.z = mt->y;
	mt->wallIdx = 0; mt->wallLoopFlag = 1;
      }
      wl = &(mt->wall[mt->wallIdx]);
      if ( wl->d == -1 ) {
	if ( !mt->wallLoopFlag ) {
	  mt->wall[mt->wallIdx+1].pt[0]->a.y = 0;
	  mt->wall[mt->wallIdx+1].pt[1]->a.y = WALL_HEIGHT;
	}
	wl->ln[0]->color = 3;
	wl->ln[1]->color = 3;
      }
      wl->d = mt->d;
      updateWallUSB(i, mt->wallIdx, mt->x, mt->y, mt->d);
      mt->wallIdx++;
      if ( mt->wallLoopFlag ) {
	wl = &(mt->wall[mt->wallIdx]);
	wl->d = -1;
	wl->ln[0]->color = NOT_EXIST;
	wl->ln[1]->color = NOT_EXIST;
	destroyWallUSB(i, mt->wallIdx);
      }
    }

    // Update the motor position.
    switch ( mt->d ) {
    case 0:
      mx = 0;
      my = mt->speed;
      break;
    case 1:
      mx = mt->speed;
      my = 0;
      break;
    case 2:
      mx = 0;
      my = -mt->speed;
      break;
    case 3:
      mx = -mt->speed;
      my = 0;
      break;
    }
    mt->x += mx; mt->y += my;
    wl = &(mt->wall[mt->wallIdx]);
    wl->pt[0]->a.x = wl->pt[1]->a.x = mt->x;
    wl->pt[0]->a.z = wl->pt[1]->a.z = mt->y;

    if ( randN(4) == 0 ) {
      addFrag(mt->x, 0, mt->y,
	      (-mx+randNS(128))>>1, randN(128), (-my+randNS(128))>>1, 
	      12+randN(12), 3);
    }

    // Play BGM and set a volume.
    if ( i != 0 ) {
      if ( playBgmTick ) playSe(mt->bgm, i);
      if ( (cnt%3) == 0 ) {
	dst =(absN(mt->x - motor[0].x) + absN(mt->y - motor[0].y))>>8;
	if ( dst <= 0 ) dst = 1;
	dst = 2500 / dst;
	if ( dst > 48 ) dst = 48;
	if ( demoFlag ) dst -= (dst>>2);
	pceWaveSetChAtt(i, 50 - dst);
      }
    }
  }
  dstWlFlag = 0;
  if ( dstWlSet ) {
    dstWlFlag = 1; dstWlSet = 0;
  }
}

void moveFrags() {
  int i;
  Frag *fr;
  for ( i=0 ; i<FRAG_MAX ; i++ ) {
    if ( frag[i].cnt <= 0 ) continue;
    fr = &(frag[i]);
    fr->cnt--;
    if ( fr->cnt <= 0 ) {
      fr->dot->p->a.x = NOT_EXIST;
      fr->dot->size = NOT_EXIST;
      continue;
    }
    fr->dot->p->a.x += fr->mx;
    fr->dot->p->a.y += fr->my;
    fr->dot->p->a.z += fr->mz;
    fr->dot->size = fr->size + randNS(3);
  }
}

// Move an eye position in demo.
void moveEye(int pad) {
  if ( pad & PAD_UP ) {
    eyeDist -= 8; if ( eyeDist < 48 ) eyeDist = 48;
  } else if ( pad & PAD_DN ) {
    eyeDist += 8; if ( eyeDist > 256 ) eyeDist = 256;
  }
}

static void drawObject(PIECE_BMP *bmp, int dx, int dy, int w, int h) {
  DRAW_OBJECT obj;
  pceLCDSetObject(&obj, bmp, dx, dy, 0, 0, w, h, DRW_NOMAL);
  pceLCDDrawObject(obj);
}

#define NUM_WIDTH 12

static void drawNum(int n, int x, int y) {
  int i;
  if ( n >= 100 ) {
    x += NUM_WIDTH-4; i = 3;
  } else if ( n >= 10 ) {
    x += (NUM_WIDTH-8)/2; i = 2;
  } else {
    x -= 4; i = 1;
  }
  for ( ; i>0 ; i-- ) {
    drawObject(&numBmp[n%10], x, y, 8, 16);
    n /= 10; x -= NUM_WIDTH;
  }
}

static int sd = 0;

void drawScene(int pad) {
  int d;
  int sc;

  flushOneSendDataBuf();

  if ( !demoFlag ) {
    d = motor[0].d*64+eyeDeg;
  } else {
    d = eyeDeg;
  }
  d -= sd;
  if ( d < -128 ) d += 256;
  if ( d >  128 ) d -= 256;
  sd += d>>1; sd &= 255;

  eyePos.x = motor[0].x - (sctbl[sd]*eyeDist);
  eyePos.z = motor[0].y - (sctbl[sd+64]*eyeDist);
  eyePos.y = eyeDist*91 + (playerDeadCnt+1)*96;
  renderAll(&eyePos, sd);

  if ( !demoFlag ) {
    if ( playerDeadCnt >= 0 ) {
      // Player crashed, Gameover.
      drawObject(&beatBmp, 16, 26, 32, 8);
      drawNum(score, 64, 36);
      drawObject(&oppBmp, 80, 54, 40, 8);
      playerDeadCnt++;
      if ( playerDeadCnt > 360 || (pad & TRG_C) ) {
	initDemo();
	return;
      }
      eyeDeg += 2; eyeDeg &= 255;
      eyeDist++;
    } else {
      // Draw a shield meter.
      if ( shield >= SHIELD_MAX || (cnt&7) < 4 ) sc = 3;
      else sc = 1;
      pceLCDPaint(sc, 14, 80, shield>>8, 4);

      if ( eyeDist > DEFAULT_EYE_DIST ) {
	eyeDist -= 16;
      }
    }
  } else {
    // Draw title.
    drawObject(&titleBmp, 96, 72, 32, 16);
    if ( (cnt&31) < 16 ) {
      drawObject(&startBmp, 0, 80, 48, 8);
    }

    eyeDeg += 1; eyeDeg &= 255;
  }
  gameCnt++;
}
